<?php
namespace Controllers;

// Контроллер для чеков
class Receipts extends Controller {
	public $products, $changes, $receiptlines;

	public function __construct() {
		parent::__construct();
		$this->model=new \Models\Receipts($this->db, $this->f3);
		$this->receiptlines=new \Models\Receiptlines($this->db, $this->f3);
		$this->products=new \Models\Products($this->db);
		$this->changes=new \Models\Changes($this->db, $this->f3);
	}

	// Раздел продаж
	function sales() {
		$this->f3->set('page_data', $this->model->page_data('sales'));
	}

	// Раздел возрата
	function return() {
		$id=$this->f3->get('GET.id');
		if(!$id) {
			$this->sales();
			$this->f3->set('page_data["title"]', $this->model->page_data('return')['title']);
			$this->f3->set('return', 1);
		}
		else {
			$receipt=$this->model->select_receipts(array('r.id=? AND r.type=1', $id), array('limit'=>1));
			if($receipt) {
				$p=new \Models\Paymentmethods($this->db);
				$paymentmethods=$p->select_method(null);
				foreach ($paymentmethods as $n => $p) {
					$pays[$p['id']]=$p['title'];
				}
				$this->f3->set('paymentmethods', $pays);
				$this->f3->set('receipt', $receipt[0]);
				$this->f3->set('page_data', $this->model->page_data('return'));
			}
			else {
				$this->f3->set('page_data', $this->messages->message('not_found'));
			}
		}
	}

	// Поиск чеков
	function search() {
		$id=$this->f3->get('GET.id');
		$changes=$this->f3->get('GET.changes');
		$barcode=$this->f3->get('GET.barcode');
		$dates=$this->f3->get('GET.dates');
		$type=$this->f3->get('GET.type');
		if($dates) {
			$dates=str_replace(' ', '', $dates);
			$dates=explode('-', $dates);
			$dates[0]=strtotime($dates[0]);
			$dates[1]=strtotime($dates[1]);
		}
		$limit=$this->f3->get('lines_of_page');
		$page=$this->f3->get('GET.page');
		if(!$page)$page=1;
		$receipts=$this->model->search($id, $changes, $barcode, $dates, $type, ($page-1)*$limit, $limit);
		$this->f3->set('pagination', $this->pagination($receipts['count'], $page));
		$this->f3->set('receipts', $receipts['receipts']);
		$this->f3->set('page_data', $this->model->page_data('search'));
	}

	// Просмотр чека
	function show() {
		$id=$this->f3->get('GET.id');
		$page_data=$this->messages->message('not_found');
		if($id) {
			$receipts=$this->model->select_receipts(array('r.id=?', $id), array('limit'=>1));
			if($receipts) {
				$p=new \Models\Paymentmethods($this->db);
				$paymentmethods=$p->select_method();
				foreach ($paymentmethods as $n => $p) {
					$pays[$p['id']]=$p['title'];
				}
				$this->f3->set('paymentmethods', $pays);
				$fr=new \Models\Fr($this->db, $this->f3);
				$sets=$fr->get_fr();
				$this->f3->set('fr', $sets);
				$this->f3->set('receipt', $receipts[0]);
				$page_data=$this->model->page_data('show');
			}
		}
		$this->f3->set('page_data', $page_data);
	}

	// добавление строки в чек
	function add_line() {
		$result=array(
			'success'=>false,
			'txt'=>$this->messages->message('not_found')['txt']
		);
		$product_id=$this->f3->get('GET.product_id');
		$count=$this->f3->get('GET.count');
		if(!$count) $count=1;
		if($product_id) {
			if($product_id>0) {
				$res=$this->products->select_product(array('p.id=?', $product_id), array('limit'=>1));
			}
			else {
				$res[0]=array(
					'title'=>'',
					'price'=>$this->f3->get('GET.price'),
					'vat'=>0,
					'vat_type'=>0
				);
			}
			if(!empty($res)) {
				$result=$this->receiptlines->add_line($res[0], $count);
				$result['txt']=$this->messages->message($result['txt'])['txt'];
			}
		}

		echo json_encode($result);
		exit();
	}

	// Меняем строку в чеке
	function change_line() {
		$line_id=$this->f3->get('REQUEST.line_id');
		$count=$this->f3->get('REQUEST.count');
		$price=$this->f3->get('REQUEST.price');
		if($line_id>=0 && ($count>=0 || $price>=0)) {
			$res=$this->receiptlines->change_line($line_id, $count, $price);
			$res['txt']=$this->messages->message($res['txt'])['txt'];
		}
		else {
			$res=array(
				'success'=>false,
				'txt'=>$this->messages->message('not_enough_data')['txt']
			);
		}

		echo json_encode($res);
		exit();
	}

	// Редактируем цену в чеке
	function change_price() {
		$discounts=array('0', '5', '10', '15', '20');
		$this->f3->set('discounts', $discounts);
		$line_id=$this->f3->get('GET.line_id');
		$lines=$this->f3->get('SESSION.receiptlines');
		if($lines[$line_id]) {
			$this->f3->set('line', $lines[$line_id]);
			$this->f3->set('page_data', $this->model->page_data('change_price'));
		}
		else {
			$this->f3->set('page_data', $this->messages->message('not_enough_data'));
		}
	}

	// Сохраняем новую цену продажи
	function post_change_price() {
		$this->change_line();
	}

	// Удаляем строки
	function delete_line() {
		$line_id=$this->f3->get('GET.line_id');
		if($line_id>=0 || $line_id=='all') {
			$res=$this->receiptlines->delete_line($line_id);
			$res['txt']=$this->messages->message($res['txt'])['txt'];
		}
		else {
			$res=array(
				'success'=>false,
				'txt'=>$this->messages->message('not_enough_data')['txt']
			);
		}

		echo json_encode($res);
		exit();
	}

	// выбор оплаты
	function pay() {
		$lines=$this->f3->get('SESSION.receiptlines');
		if(!$lines) {
			$this->f3->set('page_data', $this->messages->message('not_enough_data'));
			return;
		}
		$discount=$this->f3->get('COOKIE.discount');
		$sum=$this->receiptlines->get_sum($lines, $discount);
		$this->f3->set('sum', $sum);
		$p=new \Models\Paymentmethods($this->db);
		$paymentmethods=$p->select_method();
		$type=$this->f3->get('GET.type');
		if($type!=2) $type=1;
		$this->f3->set('type', $type);
		$this->f3->set('paymentmethods', $paymentmethods);
		$this->f3->set('page_data', $this->model->page_data('paymentmethods'));
	}

	// Пробиваем чек
	function ready($lines) {
		$paymentmethod=$this->f3->get('REQUEST.paymentmethod');
		$client_sum=$this->f3->get('REQUEST.client_sum');
		$client=$this->f3->get('REQUEST.client');
		if($lines && $paymentmethod && $client_sum) {
			$discount=$this->f3->get('COOKIE.discount');
			$partner_id=$this->f3->get('COOKIE.partner_id');
			$sum=$this->receiptlines->get_sum($lines, $discount);
			$type=$this->f3->get('REQUEST.type');
			if(!$type) $type=1; // Продажа
			// Узнаем смену
			$change=$this->changes->get_change();
			if(!empty($change)) {
				// Создаем чек
				$api=$this->controller->get_api_config();
				$res=$this->model->ready($api, $change['id'], $type, $paymentmethod, $client_sum, $sum, $lines, $client, $discount, $partner_id);
				// Добавляем товары в чек
				if($res['success']) {
					$id=$res['id'];
					$this->model->tickets_end($id);
					$res=$this->receiptlines->save_lines($id, $lines, $discount);
					// Если не удалось добавить товар, то удаляем весь чек
					if(!$res['success']) {
						$this->model->delete_receipt($id);
					}
					elseif($this->f3->get('SESSION.receiptlines')) {
						$res=$this->receiptlines->delete_line('all');
					}
				}
				$res['txt']=$this->messages->message($res['txt'])['txt'];
			}
			else {
				$res=array(
					'success'=>false,
					'txt'=>$this->messages->message('change_not_open')['txt']
				);
			}
		}
		else {
			$res=array(
				'success'=>false,
				'txt'=>$this->messages->message('not_enough_data')['txt']
			);
		}

		echo json_encode($res);
		exit();
	}

	// Создаем чек продажи
	function post_sales() {
		$lines=$this->f3->get('SESSION.receiptlines');
		$this->ready($lines);
	}

	// Создаем возвратный чек
	function post_return() {
		$post=$this->f3->get('POST');
		$this->ready($post['receiptlines']);
	}

	// Внесение денег в кассу
	function cashin() {
		$this->f3->set('page_data', $this->model->page_data('cashin'));
	}

	// Изъятие денег из кассы
	function cashout() {
		$this->f3->set('page_data', $this->model->page_data('cashout'));
	}

	// Пытаемся внести деньги в кассу
	function post_cashin() {
		$sum=$this->f3->get('POST.sum');
		if($sum) {
			$change=$this->changes->get_change();
			if($change) {
				$api=$this->controller->get_api_config();
				$p=new \Models\Paymentmethods($this->db);
				$pay=$p->select_method('cash=1');
				$res=$this->model->cashin($api, $change['id'], $pay[0]['id'],  $sum);
				$res['txt']=$this->messages->message($res['txt'])['txt'];
			}
			else {
				$res=array(
					'success'=>false,
					'txt'=>$this->messages->message('change_not_open')['txt']
				);
			}
		}
		else {
			$res=array(
				'success'=>false,
				'txt'=>$res=$this->messages->message('not_enough_data')['txt']
			);
		}

		echo json_encode($res);
		exit();
	}

	// Изымаем деньги из кассы
	function post_cashout() {
		$this->f3->set('POST.sum', -$this->f3->get('POST.sum'));
		$this->post_cashin();
	}

	// Страница смены и отчеты
	function reports() {
		$ch=$this->f3->get('GET.change');
		if(!$ch) {
			$change=$this->changes->get_change(false);
		}
		else {
			$ch=$this->changes->load(array('id=?', $ch));
			if($ch) {
				$change=$ch->cast();
			}
			else {
				$this->f3->set('page_data', $this->messages->message('not_found'));
			}
		}
		if($change) {
			$report=$this->model->change_statistic($change['id']);
			$p=new \Models\Paymentmethods($this->db);
			$paymentmethods=$p->select_method();
			foreach ($paymentmethods as $p) {
				$pays[$p['id']]=array('title'=>$p['title'], 'cash'=>$p['cash']);
			}
			$this->f3->set('paymentmethods', $pays);
		}
		$report=$this->model->change_statistic($change['id']);
		$this->f3->set('report', $report);
		$this->f3->set('change', $change);
		$this->f3->set('page_data', $this->model->page_data('reports'));
	}

	// Убрать клиента из чека
	function remove_partner() {
		$p=new \Models\Partners($this->db, $this->f3);
		$res=$p->remove_partner();
		echo json_encode($res);
		exit();
	}
}